@@ -1,4 +1,4 @@
-# Copyright 2009-2014 INRIA Rhone-Alpes, Service Experimentation et
+# Copyright 2009-2021 INRIA Rhone-Alpes, Service Experimentation et
 # Developpement
 #
 # This file is part of Execo.
@@ -16,13 +16,18 @@
 # You should have received a copy of the GNU General Public License
 # along with Execo.  If not, see <http://www.gnu.org/licenses/>
 
-from log import style, logger, logger_handler
-from time_utils import format_unixts
-from utils import compact_output
-from config import configuration
-import Queue, errno, fcntl, logging, os, select, \
-  signal, sys, thread, threading, time, traceback, \
-  subprocess, resource
+from __future__ import print_function
+from .log import style, logger, logger_handler
+from .time_utils import format_unixts
+from .utils import compact_output, MAXFD
+from .config import configuration
+import errno, fcntl, logging, os, select, \
+  signal, sys, threading, time, traceback, \
+  subprocess, resource, heapq
+if sys.version_info >= (3,):
+    import queue, _thread
+else:
+    import Queue as queue, thread as _thread
 
 # assuming import of this module is triggered from "the main" thread,
 # the following call is intended to workaround python issue #7980
@@ -35,12 +40,10 @@ except:
 
 # max number of bytes read when reading asynchronously from a pipe
 try:
-    _MAXREAD = int(subprocess.Popen(["getconf", "_POSIX_SSIZE_MAX"], stdout=subprocess.PIPE).communicate()[0])
+    _MAXREAD = int(subprocess.Popen(["getconf", "_POSIX_SSIZE_MAX"], stdout=subprocess.PIPE, universal_newlines=True).communicate()[0])
 except:
     _MAXREAD = 32767
 
-DEFAULT_MAXFD = 1024
-
 if sys.platform.startswith('darwin') or sys.platform.startswith('win'):
 
     # minimal abstraction of poll over select allowing the conductor
@@ -58,7 +61,7 @@ if sys.platform.startswith('darwin') or sys.platform.startswith('win'):
     POLLNVAL=32
 
     def dict_item_and(dic, item, field):
-        if dic.has_key(item):
+        if item in dic:
             dic[item] |= field
         else:
             dic[item] = field
@@ -110,7 +113,7 @@ if sys.platform.startswith('darwin') or sys.platform.startswith('win'):
             for fd in ready_readfds: dict_item_and(fd_events, fd, POLLIN)
             for fd in ready_writefds: dict_item_and(fd_events, fd, POLLOUT)
             for fd in ready_exceptfds: dict_item_and(fd_events, fd, POLLERR)
-            return list(fd_events.iteritems())
+            return list(fd_events.items())
 
 else:
     from select import poll, POLLIN, POLLPRI, POLLOUT, POLLERR, POLLHUP, POLLNVAL
@@ -139,7 +142,7 @@ def _checked_waitpid(pid, options):
     while True:
         try:
             return os.waitpid(pid, options)
-        except OSError, e:
+        except OSError as e:
             if e.errno == errno.ECHILD:
                 return 0, 0
             elif e.errno == errno.EINTR:
@@ -148,29 +151,29 @@ def _checked_waitpid(pid, options):
                 raise
 
 def _read_asmuch(fileno):
-    """Read as much as possible from a file descriptor withour blocking.
+    """Read as much as possible from a file descriptor without blocking.
 
     Relies on the file descriptor to have been set non blocking.
 
     Returns a tuple (string, eof). string is the data read, eof is
     a boolean flag.
     """
+    buf = b''
     eof = False
-    string = ""
     while True:
         try:
-            tmpstring = os.read(fileno, _MAXREAD)
-        except OSError, err:
+            tmpbuf = os.read(fileno, _MAXREAD)
+        except OSError as err:
             if err.errno == errno.EAGAIN:
                 break
             else:
                 raise
-        if tmpstring == "":
+        if len(tmpbuf) == 0:
             eof = True
             break
         else:
-            string += tmpstring
-    return (string, eof)
+            buf += tmpbuf
+    return (buf, eof)
 
 def _set_fd_nonblocking(fileno):
     """Sets a file descriptor in non blocking mode.
@@ -181,6 +184,18 @@ def _set_fd_nonblocking(fileno):
     fcntl.fcntl(fileno, fcntl.F_SETFL, status_flags | os.O_NONBLOCK)
     return status_flags
 
+def remove_from_heapq(hq, pred):
+    # remove all objects satisfying predicate pred from heapq hq in O(log(n))
+    i = 0
+    while i<len(hq):
+        if pred(hq[i]):
+            hq[i] = hq[-1]
+            hq.pop()
+            if i < len(hq):
+                heapq._siftup(hq, i)
+            continue
+        i += 1
+
 class _Conductor(object):
 
     """Manager of the subprocesses outputs and lifecycle.
@@ -221,6 +236,8 @@ class _Conductor(object):
                                             # _read_asmuch() relies on
                                             # file descriptors to be non
                                             # blocking
+        _set_fd_nonblocking(self.__wpipe)   # because we call
+                                            # signal.set_wakeup_fd on this pipe
         self.__poller = poll()   # asynchronous I/O with all
                                  # subprocesses filehandles
         self.__poller.register(self.__rpipe,
@@ -238,8 +255,8 @@ class _Conductor(object):
                                 # launched by this `_Conductor`
                                 #
                                 # values: their `Process`
-        self.__timeline = [] # list of `Process` with a timeout date
-        self.__process_actions = Queue.Queue()
+        self.__timeline = [] # heapq of `Process` with a timeout date
+        self.__process_actions = queue.Queue()
                                 # thread-safe FIFO used to send requests
                                 # from main thread and conductor thread:
                                 # we enqueue tuples (function to call,
@@ -266,10 +283,7 @@ class _Conductor(object):
             os.umask(0)
             maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
             if (maxfd == resource.RLIM_INFINITY):
-                if (os.sysconf_names.has_key("SC_OPEN_MAX")):
-                    maxfd = maxfd = os.sysconf("SC_OPEN_MAX")
-                else:
-                    maxfd = DEFAULT_MAXFD
+                maxfd = MAXFD
             for fd in range(0, maxfd):
                 try: os.close(fd)
                 except OSError: pass
@@ -286,7 +300,7 @@ class _Conductor(object):
 
     def __wakeup(self):
         # wakeup the I/O thread
-        os.write(self.__wpipe, ".")
+        os.write(self.__wpipe, b'.')
 
     def start(self):
         """Start the conductor thread."""
@@ -299,6 +313,8 @@ class _Conductor(object):
         # detect this closing and self stop
         logger.debug("terminating I/O thread of %s", self)
         os.close(self.__wpipe)
+        self.__io_thread.join()
+        logger.debug("I/O thread of %s terminated", self)
 
     def start_process(self, process):
         """Register a new `execo.process.Process` to be started and handled by the conductor.
@@ -313,9 +329,7 @@ class _Conductor(object):
 
         Intended to be called from main thread.
 
-        Currently: only update the timeout. This is related to the way
-        the system for forcing SIGKILL on processes not killing
-        cleanly is implemented.
+        Currently: only update the force kill timeout.
         """
         self.__process_actions.put_nowait((self.__handle_update_process, (process,)))
         self.__wakeup()
@@ -356,7 +370,7 @@ class _Conductor(object):
                                    | POLLERR)
             self.__pids[process.pid] = process
             if process.timeout_date != None:
-                self.__timeline.append((process.timeout_date, process))
+                heapq.heappush(self.__timeline,(process.timeout_date, process))
             if self.__reaper_thread_running == False:
                 self.__reaper_thread_running = True
                 reaper_thread = threading.Thread(target = self.__reaper_thread_func, name = "Reaper")
@@ -365,50 +379,48 @@ class _Conductor(object):
 
     def __handle_update_process(self, process):
         # intended to be called from conductor thread
-        # Currently: only update the timeout. This is related to the
-        # way the system for forcing SIGKILL on processes not killing
-        # cleanly is implemented.
-        logger.fdebug("update timeouts of %s in %s", str(process), self)
+        # Currently: only update the force kill timeout.
+        logger.fdebug("update force kill timeout of %s in %s", str(process), self)
         if process not in self.__processes:
             return  # this will frequently occur if the process kills
                     # quickly because the process will already be
                     # killed and reaped before __handle_update_process
                     # is called
-        if process.timeout_date != None:
-            self.__timeline.append((process.timeout_date, process))
+        if process._force_kill_timeout_date != None:
+            heapq.heappush(self.__timeline,(process._force_kill_timeout_date, process))
 
     def __handle_remove_process(self, process, exit_code = None):
         # intended to be called from conductor thread
         # unregister a Process from conductor
         logger.fdebug("removing %s from %s", str(process), self)
         if process not in self.__processes:
-            raise ValueError, "trying to remove a process which was not yet added to conductor"
-        self.__timeline = [ x for x in self.__timeline if x[1] != process ]
+            raise ValueError("trying to remove a process which was not yet added to conductor")
+        remove_from_heapq(self.__timeline, lambda e: e[1] == process)
         del self.__pids[process.pid]
         fileno_stdout = process.stdout_fd
         fileno_stderr = process.stderr_fd
-        last_bytes = ""
-        if self.__fds.has_key(fileno_stdout):
+        last_bytes = b''
+        if fileno_stdout in self.__fds:
             del self.__fds[fileno_stdout]
             self.__poller.unregister(fileno_stdout)
             # read the last data that may be available on stdout of
             # this process
             try:
                 (last_bytes, _) = _read_asmuch(fileno_stdout)
-            except OSError, e:
-                if e.errno == errno.EBADF: last_bytes = ""
+            except OSError as e:
+                if e.errno == errno.EBADF: last_bytes = b''
                 else: raise e
         process._handle_stdout(last_bytes, True, False)
-        last_bytes = ""
-        if self.__fds.has_key(fileno_stderr):
+        last_bytes = b''
+        if fileno_stderr in self.__fds:
             del self.__fds[fileno_stderr]
             self.__poller.unregister(fileno_stderr)
             # read the last data that may be available on stderr of
             # this process
             try:
                 (last_bytes, _) = _read_asmuch(fileno_stderr)
-            except OSError, e:
-                if e.errno == errno.EBADF: last_bytes = ""
+            except OSError as e:
+                if e.errno == errno.EBADF: last_bytes = b''
                 else: raise e
         process._handle_stderr(last_bytes, True, False)
         self.__processes.remove(process)
@@ -418,28 +430,24 @@ class _Conductor(object):
     def __get_next_timeout(self):
         """Return the remaining time until the smallest timeout date of all registered `execo.process.Process`."""
         next_timeout = None
-        if len(self.__timeline) != 0:
-            self.__timeline.sort(key = lambda x: x[0])
-            next_timeout = (self.__timeline[0][0] - time.time())
+        if len(self.__timeline) > 0:
+            next_timeout = self.__timeline[0][0] - time.time()
         return next_timeout
 
     def __check_timeouts(self):
-        """Iterate all registered `execo.process.Process` whose timeout is reached, kill them gently.
+        """Iterate all registered `execo.process.Process` whose timeout is reached, kill them.
 
         And remove them from the timeline.
         """
         now = time.time()
-        remove_in_timeline = []
-        for i in xrange(0, len(self.__timeline)):
-            process = self.__timeline[i][1]
-            if now >= process.timeout_date:
-                logger.debug("timeout on %s", str(process))
+        while len(self.__timeline) > 0 and self.__timeline[0][0] <= now:
+            _, process = heapq.heappop(self.__timeline)
+            if process._force_kill_timeout_date != None and now >= process._force_kill_timeout_date:
+                logger.debug("force kill timeout on %s" % (str(process),))
+                process._force_kill()
+            elif now >= process.timeout_date:
+                logger.debug("timeout on %s" % (str(process),))
                 process._timeout_kill()
-                remove_in_timeline.append(i)
-            else:
-                break
-        for j in reversed(remove_in_timeline):
-            del(self.__timeline[j])
 
     def __update_terminated_processes(self):
         """Ask operating system for all processes that have terminated, self remove them.
@@ -472,9 +480,9 @@ class _Conductor(object):
         try:
             self.__io_loop()
         except Exception: #IGNORE:W0703
-            print "exception in conductor I/O loop thread"
+            print("exception in conductor I/O loop thread")
             traceback.print_exc()
-            thread.interrupt_main()
+            _thread.interrupt_main()
 
     def __io_loop(self):
         # conductor thread infinite I/O loop
@@ -500,7 +508,7 @@ class _Conductor(object):
                 if fd == self.__rpipe:
                     event_on_rpipe = event
                 else:
-                    if self.__fds.has_key(fd):
+                    if fd in self.__fds:
                         process, stream_handler_func = self.__fds[fd]
                         logger.fdebug("event %s on fd %s, process %s", _event_desc(event), fd, str(process))
                         if event & POLLIN:
@@ -526,21 +534,28 @@ class _Conductor(object):
                     finished = True
                 if event_on_rpipe & POLLERR:
                     finished = True
-                    raise IOError, "Error on inter-thread communication pipe"
+                    raise IOError("Error on inter-thread communication pipe")
             with self.lock:
                 while True:
                     try:
                         # call (in the right order!) all functions
                         # enqueued from other threads
                         func, args = self.__process_actions.get_nowait()
-                    except Queue.Empty:
+                    except queue.Empty:
                         break
                     func(*args)
                 self.__update_terminated_processes()
                 self.condition.notifyAll()
+        logger.debug("conductor exiting I/O loop")
         self.__poller.unregister(self.__rpipe)
-        os.close(self.__rpipe)
-        os.close(self.__wpipe)
+        try:
+            os.close(self.__rpipe)
+        except:
+            pass
+        try:
+            os.close(self.__wpipe)
+        except:
+            pass
 
     def __reaper_thread_func(self):
         # run func for the reaper thread, whose role is to wait to be
@@ -584,33 +599,33 @@ the_conductor = _Conductor().start()
 
 def debug_dump_processes():
     with the_conductor.lock:
-        print >> sys.stderr, "\n===== %s dump current %i conductor handled processes:\n" % (format_unixts(time.time()), len(the_conductor._Conductor__processes),)
+        print("\n===== %s dump current %i conductor handled processes:\n" % (format_unixts(time.time()), len(the_conductor._Conductor__processes),), file=sys.stderr)
         for process in the_conductor._Conductor__processes:
-            print >> sys.stderr, "====="
-            print >> sys.stderr, str(process)
-            print >> sys.stderr, "stdout:\n" + compact_output(process.stdout)
-            print >> sys.stderr, "stderr:\n" + compact_output(process.stderr)
-            print >> sys.stderr
-        print >> sys.stderr, "=====\n"
+            print("=====", file=sys.stderr)
+            print(str(process), file=sys.stderr)
+            print("stdout:\n" + compact_output(process.stdout), file=sys.stderr)
+            print("stderr:\n" + compact_output(process.stderr), file=sys.stderr)
+            print(file=sys.stderr)
+        print("=====\n", file=sys.stderr)
 
 _debug_thread = None
 _debug_thread_id = None
 _debug_thread_lock = threading.Lock()
 
 def debug_dump_threads():
-    print >> sys.stderr, "\n===== %s dump thread stack frames. %i threads. conductor lock = %s:\n" % (
+    print("\n===== %s dump thread stack frames. %i threads. conductor lock = %s:\n" % (
         format_unixts(time.time()),
         len(sys._current_frames()),
-        the_conductor.lock)
+        the_conductor.lock), file=sys.stderr)
     idx=0
-    for thread_id, frame in sys._current_frames().iteritems():
-        print >> sys.stderr, "===== thread #%i [%#x] refcount = %s" % (idx, thread_id, sys.getrefcount(frame))
+    for thread_id, frame in sys._current_frames().items():
+        print("===== thread #%i [%#x] refcount = %s" % (idx, thread_id, sys.getrefcount(frame)), file=sys.stderr)
         if thread_id != _debug_thread_id:
             traceback.print_stack(frame, file = sys.stderr)
         else:
-            print >> sys.stderr, "  debug thread, skipping"
+            print("  debug thread, skipping", file=sys.stderr)
         idx += 1
-    print >> sys.stderr, "=====\n"
+    print("=====\n", file=sys.stderr)
 
 class _DebugThread(threading.Thread):
 
