@@ -1,4 +1,4 @@
-# Copyright 2009-2014 INRIA Rhone-Alpes, Service Experimentation et
+# Copyright 2009-2021 INRIA Rhone-Alpes, Service Experimentation et
 # Developpement
 #
 # This file is part of Execo.
@@ -16,6 +16,7 @@
 # You should have received a copy of the GNU General Public License
 # along with Execo.  If not, see <http://www.gnu.org/licenses/>
 
+from __future__ import print_function
 import logging
 import os
 import sys
@@ -51,7 +52,7 @@ configuration = {
     'fileput_tool': SCP,
     'fileget_tool': SCP,
     'compact_output_threshold': 4096,
-    'kill_timeout': 5,
+    'kill_timeout': 60,
     'intr_period': 1,
     'port_range': (25500, 26700),
     'kill_childs_at_end': True,
@@ -159,6 +160,7 @@ def make_default_connection_params():
         'chainput_host_retry': 5,
         'chainput_chain_retry': 10,
         'chainput_try_delay': 1,
+        'forwarding_timeout': 25,
         'pty': False,
         'host_rewrite_func': None
         }
@@ -210,6 +212,9 @@ default_connection_params = make_default_connection_params()
 - ``chainput_try_delay``: delay in seconds between TCP client to
   server connection attempts.
 
+- ``forwarding_timeout``: max time to wait for a forwarding
+  port to be available
+
 - ``pty``: boolean. Wether to allocate or not a pty for
   ssh/scp.
 
@@ -240,21 +245,26 @@ def load_configuration(filename, dicts_confs):
     if os.path.isfile(filename):
         jailed_globals = {}
         try:
-            execfile(filename, jailed_globals)
-        except Exception, exc: #IGNORE:W0703
-            print "ERROR while reading config file %s:" % (filename,)
-            print exc
+            with open(filename) as f:
+                code = f.read()
+                ccode = compile(code, filename, 'exec')
+                exec(ccode, jailed_globals)
+        except Exception as exc: #IGNORE:W0703
+            print("ERROR while reading config file %s:" % (filename,))
+            print(exc)
         for (dictio, conf) in dicts_confs:
-            if jailed_globals.has_key(conf):
+            if conf in jailed_globals:
                 dictio.update(jailed_globals[conf])
 
 def get_user_config_filename():
     _user_conf_file = None
-    if os.environ.has_key('HOME'):
+    if 'HOME' in os.environ:
         _user_conf_file = os.environ['HOME'] + '/.execo.conf.py'
     return _user_conf_file
 
-load_configuration(
-  get_user_config_filename(),
-  ((configuration, 'configuration'),
-   (default_connection_params, 'default_connection_params')))
+__cf = get_user_config_filename()
+if __cf:
+    load_configuration(
+        __cf,
+        ((configuration, 'configuration'),
+         (default_connection_params, 'default_connection_params')))
