@@ -1,4 +1,4 @@
-# Copyright 2009-2014 INRIA Rhone-Alpes, Service Experimentation et
+# Copyright 2009-2021 INRIA Rhone-Alpes, Service Experimentation et
 # Developpement
 #
 # This file is part of Execo.
@@ -17,7 +17,7 @@
 # along with Execo.  If not, see <http://www.gnu.org/licenses/>
 
 
-from log import logger
-from engine import Engine
-from utils import slugify, redirect_outputs, copy_outputs
-from sweep import HashableDict, sweep, ParamSweeper, geom, igeom, sweep_stats
+from .log import logger
+from .engine import Engine
+from .utils import slugify, redirect_outputs, copy_outputs
+from .sweep import HashableDict, sweep, ParamSweeper, geom, igeom, sweep_stats
