@@ -1,4 +1,4 @@
-# Copyright 2009-2014 INRIA Rhone-Alpes, Service Experimentation et
+# Copyright 2009-2021 INRIA Rhone-Alpes, Service Experimentation et
 # Developpement
 #
 # This file is part of Execo.
@@ -16,58 +16,13 @@
 # You should have received a copy of the GNU General Public License
 # along with Execo.  If not, see <http://www.gnu.org/licenses/>
 
-from log import logger
-import optparse, os, sys, time, inspect, pipes
-from utils import redirect_outputs, copy_outputs
+from .log import logger
+import os, sys, time, inspect, pipes
+from .utils import redirect_outputs, copy_outputs
+from argparse import ArgumentParser
 
 _engineargs = sys.argv[1:]
 
-class ArgsOptionParser(optparse.OptionParser):
-
-    """optparse.OptionParser subclass which keeps tracks of arguments for proper help string generation.
-
-    This class is a rather quick and dirty hack. Using ``argparse``
-    would be better but it's only available in python 2.7.
-    """
-
-    def __init__(self, *args, **kwargs):
-        self.arguments = []
-        optparse.OptionParser.__init__(self, *args, **kwargs)
-
-    def add_argument(self, arg_name, description):
-        """Add an arg_name to the arg_name list."""
-        self.arguments.append((arg_name, description))
-
-    def num_arguments(self):
-        """Returns the number of expected arguments."""
-        return len(self.arguments)
-
-    def format_arguments(self, formatter=None):
-        if formatter is None:
-            formatter = self.formatter
-        result = []
-        result.append(formatter.format_heading("Arguments"))
-        formatter.indent()
-        for argument in self.arguments:
-            result.append(formatter._format_text("%s: %s" % (argument[0], argument[1])))
-            result.append("\n")
-        formatter.dedent()
-        result.append("\n")
-        return "".join(result)
-
-    def format_help(self, formatter=None):
-        if formatter is None:
-            formatter = self.formatter
-        result = []
-        if self.usage:
-            result.append(self.get_usage() + "\n")
-        if self.description:
-            result.append(self.format_description(formatter) + "\n")
-        result.append(self.format_arguments(formatter))
-        result.append(self.format_option_help(formatter))
-        result.append(self.format_epilog(formatter))
-        return "".join(result)
-
 def run_meth_on_engine_ancestors(instance, method_name):
     engine_ancestors = [ cls for cls in inspect.getmro(instance.__class__) if issubclass(cls, Engine) ]
     for cls in engine_ancestors:
@@ -101,9 +56,7 @@ class Engine(object):
 
     - `execo_engine.engine.Engine.result_dir`
 
-    - `execo_engine.engine.Engine.options_parser`
-
-    - `execo_engine.engine.Engine.options`
+    - `execo_engine.engine.Engine.args_parser`
 
     - `execo_engine.engine.Engine.args`
 
@@ -121,22 +74,33 @@ class Engine(object):
 
     - `execo_engine.engine.Engine.setup_result_dir`
 
-    A typical, non-reusable engine would override
-    `execo_engine.engine.Engine.init`, adding options / arguments to
-    `execo_engine.engine.Engine.options_parser`, and override
-    `execo_engine.engine.Engine.run`, putting all the experiment code
-    inside it, being sure that all initializations are done when
-    ``run`` is called: results directory is initialized and created
+    A typical, non-reusable engine would start by adding options /
+    arguments to `execo_engine.engine.Engine.args_parser` in `__init__()`,
+    then override `execo_engine.engine.Engine.init` to perform further
+    initialization if needed.  It would then implement all the experiment
+    code by overriding `execo_engine.engine.Engine.run`.  This ensures
+    that all initialization steps are performed by the engine before the
+    experiment runs: results directory is initialized and created
     (possibly reusing a previous results directory, to restart from a
-    previously stopped experiment), log level is set, stdout / stderr
-    are redirected as needed, and options and arguments are in
-    `execo_engine.engine.Engine.options` and
+    previously stopped experiment), log level is set, stdout / stderr are
+    redirected as needed, and options and arguments are in
     `execo_engine.engine.Engine.args`.
 
+    Example engine with custom command-line arguments::
+
+     class MyEngine(execo_engine.Engine):
+         def __init__(self):
+             super(MyEngine, self).__init__()
+             self.args_parser.add_argument('--myoption', default='foo',
+                                           help='An option to control how the experiment is done')
+         def run(self):
+             if self.args.myoption == "foo":
+                 ...
+
     A typical usage of a `execo_engine.utils.ParamSweeper` in an
-    engine would be to intialize one at the beggining of
-    `execo_engine.engine.Engine.run`, using a persistence file in the
-    results directory. example code::
+    engine would be to initialize an instance at the beginning of
+    `execo_engine.engine.Engine.run`, using a persistent file in the
+    results directory. Example code::
 
      def run(self):
          sweeps = sweep({<parameters and their values>})
@@ -160,39 +124,30 @@ class Engine(object):
         mymodule = sys.modules[self.__module__]
         if hasattr(mymodule, "__file__"):
             self.engine_dir = os.path.abspath(os.path.dirname(os.path.realpath(mymodule.__file__)))
-        self.options_parser = ArgsOptionParser(usage = "usage: <program> [options] <arguments>")
-        """An instance of
-        `execo_engine.engine.ArgsOptionParser`. Subclasses of
-        `execo_engine.engine.Engine` can register options and args to
-        this options parser in `execo_engine.engine.Engine.init`.
-        """
-        self.options_parser.add_option(
+        self.args_parser = ArgumentParser(usage = "<program> [options] <arguments>",
+                                          description = "engine: " + self.__class__.__name__)
+        """Subclasses of `execo_engine.engine.Engine` can register options and args to this options parser in `__init__()`."""
+        self.args_parser.add_argument(
             "-l", dest = "log_level", default = None,
             help = "log level (int or string). Default = inherit execo logger level")
-        self.options_parser.add_option(
+        self.args_parser.add_argument(
             "-L", dest = "output_mode", action="store_const", const = "copy", default = False,
-            help = "copy stdout / stderr to log files in the experiment result directory. Default = %default")
-        self.options_parser.add_option(
+            help = "copy stdout / stderr to log files in the experiment result directory. Default = %(default)s")
+        self.args_parser.add_argument(
             "-R", dest = "output_mode", action="store_const", const = "redirect", default = False,
-            help = "redirect stdout / stderr to log files in the experiment result directory. Default = %default")
-        self.options_parser.add_option(
+            help = "redirect stdout / stderr to log files in the experiment result directory. Default = %(default)s")
+        self.args_parser.add_argument(
             "-M", dest = "merge_outputs", action="store_true", default = False,
-            help = "when copying or redirecting outputs, merge stdout / stderr in a single file. Default = %default")
-        self.options_parser.add_option(
+            help = "when copying or redirecting outputs, merge stdout / stderr in a single file. Default = %(default)s")
+        self.args_parser.add_argument(
             "-c", dest = "use_dir", default = None, metavar = "DIR",
             help = "use experiment directory DIR")
-        self.options_parser.set_description("engine: " + self.__class__.__name__)
-        self.options = None
-        """Options given on the command line. Available after the
-        command line has been parsed, in
-        `execo_engine.engine.Engine.run` (not in
-        `execo_engine.engine.Engine.init`)
-        """
         self.args = None
-        """Arguments given on the command line. Available after the
-        command line has been parsed, in
+        """Arguments and options given on the command line. Available after
+        the command line has been parsed, in
         `execo_engine.engine.Engine.run` (not in
         `execo_engine.engine.Engine.init`)
+
         """
         self.run_name = None
         """Name of the current experiment. If you want to modify it,
@@ -207,7 +162,7 @@ class Engine(object):
         experiment should be located.
         """
 
-    def start(self):
+    def start(self, engineargs = _engineargs):
         """Start the engine.
 
         Properly initialize the experiment Engine instance, then call
@@ -215,7 +170,7 @@ class Engine(object):
         the overridden run() method of the requested experiment
         Engine.
         """
-        (self.options, self.args) = self.options_parser.parse_args(args = _engineargs)
+        self.args = self.args_parser.parse_args(args = engineargs)
         # _engineargs hack: to make running an engine from ipython
         # more convenient: if ipython is run from command line as
         #
@@ -227,32 +182,33 @@ class Engine(object):
         # instanciate or start the engine interactively, we need to
         # parse the right args from sys.argv which is thus saved at
         # engine load time.
-        if self.options.log_level != None:
+        if self.args.log_level != None:
             try:
-                log_level = int(self.options.log_level)
+                log_level = int(self.args.log_level)
             except:
-                log_level = self.options.log_level
+                log_level = self.args.log_level
             logger.setLevel(log_level)
-        if len(self.args) < self.options_parser.num_arguments():
-            self.options_parser.print_help(sys.stderr)
-            exit(1)
+        if sys.version_info < (2, 7):
+            if len(self.args) < self.args_parser.num_arguments():
+                self.args_parser.print_help(sys.stderr)
+                exit(1)
         self.setup_run_name()
-        if self.options.use_dir:
-            self.result_dir = self.options.use_dir
+        if self.args.use_dir:
+            self.result_dir = self.args.use_dir
         else:
             self.setup_result_dir()
         self._create_result_dir()
-        if self.options.output_mode:
-            if self.options.merge_outputs:
+        if self.args.output_mode:
+            if self.args.merge_outputs:
                 stdout_fname = self.result_dir + "/stdout+stderr"
                 stderr_fname = self.result_dir + "/stdout+stderr"
             else:
                 stdout_fname = self.result_dir + "/stdout"
                 stderr_fname = self.result_dir + "/stderr"
-            if self.options.output_mode == "copy":
+            if self.args.output_mode == "copy":
                 copy_outputs(stdout_fname, stderr_fname)
                 logger.info("dup stdout / stderr to %s and %s", stdout_fname, stderr_fname)
-            elif self.options.output_mode == "redirect":
+            elif self.args.output_mode == "redirect":
                 redirect_outputs(stdout_fname, stderr_fname)
                 logger.info("redirect stdout / stderr to %s and %s", stdout_fname, stderr_fname)
         logger.info("command line arguments: %s" % (sys.argv,))
