@@ -1,4 +1,4 @@
-# Copyright 2009-2014 INRIA Rhone-Alpes, Service Experimentation et
+# Copyright 2009-2021 INRIA Rhone-Alpes, Service Experimentation et
 # Developpement
 #
 # This file is part of Execo.
@@ -18,4 +18,7 @@
 
 import logging
 import execo.log #IGNORE:W0611 #@PydevCodeAnalysisIgnore #@UnusedImport
+__default_logger = logging.getLoggerClass()
+logging.setLoggerClass(execo.log.Logger)
 logger = logging.getLogger("execo.engine")
+logging.setLoggerClass(__default_logger)
