@@ -1,4 +1,4 @@
-# Copyright 2009-2014 INRIA Rhone-Alpes, Service Experimentation et
+# Copyright 2009-2021 INRIA Rhone-Alpes, Service Experimentation et
 # Developpement
 #
 # This file is part of Execo.
@@ -16,9 +16,13 @@
 # You should have received a copy of the GNU General Public License
 # along with Execo.  If not, see <http://www.gnu.org/licenses/>
 
-import threading, os, fcntl, math
-import cPickle as pickle
-from log import logger
+import threading, os, fcntl, math, sys
+import pickle as pickle
+if sys.version_info >= (3,):
+    import pickle
+else:
+    import cPickle as pickle
+from .log import logger
 
 def geom(range_min, range_max, num_steps):
     """Return a geometric progression from range_min to range_max with num_steps"""
@@ -122,7 +126,7 @@ def sweep(parameters):
     return result
 
 # context manager for opening and locking files
-# beware: for locking purpose, the file is always opened in mode "a+"
+# beware: for locking purpose, the file is always opened in mode "ab+"
 # which is the only mode allowing both locking the file and having
 # read write access to it. but it forces to handle correctly file
 # position and truncation
@@ -132,7 +136,7 @@ class _openlock():
         self.__filename = filename
 
     def __enter__(self):
-        self.__file = open(self.__filename, "a+")
+        self.__file = open(self.__filename, "ab+")
         fcntl.lockf(self.__file, fcntl.LOCK_EX)
         return self.__file
 
@@ -205,11 +209,35 @@ class ParamSweeper(object):
     used with `execo_engine.engine.Engine` startup option ``-c``
     (continue experiment in a given directory).
 
+    `execo_engine.sweep.ParamSweeper.skip` is intended to be used when
+    you got a combination with
+    `execo_engine.sweep.ParamSweeper.get_next` but the processing of
+    this combination has failed and you don't want to retry it later
+    in the same run of your script.
+
+    But the skipped state is not written to disk, if you relaunch your
+    script later, the previously skipped combinations will again be
+    iterated over.
+
+    The intent is for situations where, for example, some combinations
+    cannot be processed because, for example, there are not enough
+    available resources to process them. You may then want to relaunch
+    your script later (after making sure more resources are available)
+    to process these skipped combinations.
+
+    If there are some combinations that you never want to retry, even
+    in subsequent runs of your script, then mark them as done with
+    `execo_engine.sweep.ParamSweeper.done`.
+
+    If there is a combination that you failed to process but you want
+    to retry it in the current run of your script, mark it as canceled
+    with `execo_engine.sweep.ParamSweeper.cancel`.
+
     State *inprogress* is stored on disk to avoid concurrent processes
     to get the same elements from different ParamSweeper instances (to
     avoid duplicating work). In some cases (for example if a process
     has crashed without marking an element *done* or *skipped* or
-    cancelling it), you may want to reset the *inprogress* state. This
+    canceling it), you may want to reset the *inprogress* state. This
     can be done by removing the file ``inprogress`` in the persistence
     directory (this can even be done while some ParamSweeper are
     instanciated and using it).
@@ -224,12 +252,13 @@ class ParamSweeper(object):
     a huge number of elements to iterate on (and accordingly, the list
     of done elements will grow huge too), and that the number of
     inprogress elements will stay reasonably low. The whole iterable
-    of elements is (optionnaly) written to disk only once, at
+    of elements is (optionally) written to disk only once, at
     ParamSweeper construction. The set of done elements can only grow
     and is incrementaly appended. The set of inprogress elements is
     fully read from and written to disk at each operation, thus this
     may become a bottleneck to ParamSweeper performance if the set of
     inprogress elements is big.
+
     """
 
     def __init__(self, persistence_dir, sweeps = None, save_sweeps = False, name = None):
@@ -448,21 +477,59 @@ class ParamSweeper(object):
                     if filtr:
                         remaining = filtr(remaining)
                     try:
-                        combination = iter(remaining).next()
+                        combination = next(iter(remaining))
                     except StopIteration:
                         logger.trace("%s no new combination", self.__name)
                         logger.trace(self)
                         return None
                     self.__remaining.discard(combination)
                     self.__inprogress.add(combination)
-                    if combination in self.__sweeps:
-                        self.__filtered_inprogress.add(combination)
+                    self.__filtered_inprogress.add(combination)
                     inprogress_file.truncate(0)
                     pickle.dump(self.__inprogress, inprogress_file)
             logger.trace("%s new combination: %s", self.__name, combination)
             logger.trace(self)
             return combination
 
+    def get_next_batch(self, num_combs, filtr = None):
+        """Return the next elements which are *todo*.
+
+        :param num_combs: how much combinations to get. An array of
+          combinations is returned. The size of the array is <=
+          num_combs and is limited by the number of available
+          remaining combinations
+
+        :param filtr: a filter function. If not None, this filter
+          takes the iterable of remaining elements and returns a
+          filtered iterable. It can be used to filter out some
+          combinations and / or control the order of iteration.
+        """
+        combinations = []
+        with self.__lock:
+            with _openlock(os.path.join(self.__persistence_dir, "done")) as done_file:
+                with _openlock(os.path.join(self.__persistence_dir, "inprogress")) as inprogress_file:
+                    self.__nolock_update(done_file, inprogress_file)
+                    remaining = self.__remaining
+                    if filtr:
+                        remaining = filtr(remaining)
+                    try:
+                        for i in remaining:
+                            if num_combs <= 0:
+                                break
+                            combinations.append(i)
+                            num_combs -= 1
+                    except StopIteration:
+                        logger.trace("%s no new combination", self.__name)
+                        logger.trace(self)
+                    self.__remaining.difference_update(combinations)
+                    self.__inprogress.update(combinations)
+                    self.__filtered_inprogress.update(combinations)
+                    inprogress_file.truncate(0)
+                    pickle.dump(self.__inprogress, inprogress_file)
+            logger.trace("%s new combinations: %s", self.__name, combinations)
+            logger.trace(self)
+            return combinations
+
     def done(self, combination):
         """mark the given element *done*"""
         with self.__lock:
@@ -482,6 +549,27 @@ class ParamSweeper(object):
             logger.trace("%s combination done: %s", self.__name, combination)
             logger.trace(self)
 
+    def done_batch(self, combinations):
+        """mark the given element(s) *done*"""
+        with self.__lock:
+            with _openlock(os.path.join(self.__persistence_dir, "done")) as done_file:
+                with _openlock(os.path.join(self.__persistence_dir, "inprogress")) as inprogress_file:
+                    self.__nolock_update(done_file, inprogress_file)
+                    self.__remaining.difference_update(combinations)
+                    self.__inprogress.difference_update(combinations)
+                    self.__filtered_inprogress.difference_update(combinations)
+                    self.__done.update(combinations)
+                    filtered_combinations = set(combinations)
+                    filtered_combinations.intersection_update(self.__sweeps)
+                    self.__filtered_done.update(filtered_combinations)
+                    done_file.seek(0, os.SEEK_END)
+                    for combination in combinations:
+                        pickle.dump(combination, done_file)
+                    inprogress_file.truncate(0)
+                    pickle.dump(self.__inprogress, inprogress_file)
+            logger.trace("%s combinations done: %s", self.__name, combinations)
+            logger.trace(self)
+
     def skip(self, combination):
         """mark the given element *skipped*"""
         with self.__lock:
@@ -498,6 +586,23 @@ class ParamSweeper(object):
             logger.trace("%s combination skipped: %s", self.__name, combination)
             logger.trace(self)
 
+    def skip_batch(self, combinations):
+        """mark the given element(s) *skipped*"""
+        with self.__lock:
+            with _openlock(os.path.join(self.__persistence_dir, "done")) as done_file:
+                with _openlock(os.path.join(self.__persistence_dir, "inprogress")) as inprogress_file:
+                    self.__nolock_update(done_file, inprogress_file)
+                    self.__skipped.update(combinations)
+                    filtered_combinations = set(combinations)
+                    filtered_combinations.intersection_update(self.__sweeps)
+                    self.__filtered_skipped.update(filtered_combinations)
+                    self.__inprogress.difference_update(combinations)
+                    self.__filtered_inprogress.difference_update(combinations)
+                    inprogress_file.truncate(0)
+                    pickle.dump(self.__inprogress, inprogress_file)
+            logger.trace("%s combinations skipped: %s", self.__name, combinations)
+            logger.trace(self)
+
     def cancel(self, combination):
         """cancel processing of the given combination, but don't mark it as skipped, it comes back in the *todo* queue."""
         with self.__lock:
@@ -513,6 +618,22 @@ class ParamSweeper(object):
             logger.trace("%s combination cancelled: %s", self.__name, combination)
             logger.trace(self)
 
+    def cancel_batch(self, combinations):
+        """cancel processing of the given combination(s), but don't mark it/them as skipped, they comes back in the *todo* queue."""
+        with self.__lock:
+            with _openlock(os.path.join(self.__persistence_dir, "done")) as done_file:
+                with _openlock(os.path.join(self.__persistence_dir, "inprogress")) as inprogress_file:
+                    self.__nolock_update(done_file, inprogress_file)
+                    filtered_combinations = set(combinations)
+                    filtered_combinations.intersection_update(self.__sweeps)
+                    self.__remaining.update(filtered_combinations)
+                    self.__inprogress.difference_update(combinations)
+                    self.__filtered_inprogress.difference_update(combinations)
+                    inprogress_file.truncate(0)
+                    pickle.dump(self.__inprogress, inprogress_file)
+            logger.trace("%s combinations cancelled: %s", self.__name, combinations)
+            logger.trace(self)
+
     def reset(self, reset_inprogress = False):
         """reset container: iteration will start from beginning, state *skipped* are forgotten, state *done* are **not** forgotten.
 
@@ -555,9 +676,9 @@ def sweep_stats(stats):
         counts = dict()
         for comb in combs:
             for k in comb:
-                if not counts.has_key(k):
+                if k not in counts:
                     counts[k] = dict()
-                if not counts[k].has_key(comb[k]):
+                if comb[k] not in counts[k]:
                     counts[k][comb[k]] = 0
                 counts[k][comb[k]] += 1
         return counts
@@ -582,22 +703,22 @@ def sweep_stats(stats):
         inprogress_ratio[k1] = dict()
         done_ratio[k1] = dict()
         for k2 in ctotal[k1]:
-            if cremaining.has_key(k1) and cremaining[k1].has_key(k2):
+            if k1 in cremaining and k2 in cremaining[k1]:
                 r = cremaining[k1][k2]
             else:
                 r = 0
             remaining_ratio[k1][k2] = float(r) / float(ctotal[k1][k2])
-            if cskipped.has_key(k1) and cskipped[k1].has_key(k2):
+            if k1 in cskipped and k2 in cskipped[k1]:
                 s = cskipped[k1][k2]
             else:
                 s = 0
             skipped_ratio[k1][k2] = float(s) / float(ctotal[k1][k2])
-            if cinprogress.has_key(k1) and cinprogress[k1].has_key(k2):
+            if k1 in cinprogress and k2 in cinprogress[k1]:
                 i = cinprogress[k1][k2]
             else:
                 i = 0
             inprogress_ratio[k1][k2] = float(i) / float(ctotal[k1][k2])
-            if cdone.has_key(k1) and cdone[k1].has_key(k2):
+            if k1 in cdone and k2 in cdone[k1]:
                 d = cdone[k1][k2]
             else:
                 d = 0
