@@ -1,4 +1,4 @@
-# Copyright 2009-2014 INRIA Rhone-Alpes, Service Experimentation et
+# Copyright 2009-2021 INRIA Rhone-Alpes, Service Experimentation et
 # Developpement
 #
 # This file is part of Execo.
@@ -18,41 +18,44 @@
 
 """Execo extension and tools for Grid5000."""
 
-from config import g5k_configuration, \
+from .config import g5k_configuration, \
     default_frontend_connection_params, default_oarsh_oarcp_params
 
-from oar import OarSubmission, oarsub, oardel, get_current_oar_jobs, \
+from .oar import OarSubmission, oarsub, oardel, get_current_oar_jobs, \
     get_oar_job_info, wait_oar_job_start, get_oar_job_nodes, \
     get_oar_job_subnets, get_oar_job_kavlan, oarsubgrid
 
-from oargrid import oargridsub, oargriddel, \
+from .oargrid import oargridsub, oargriddel, \
     get_current_oargrid_jobs, get_oargrid_job_info, \
     get_oargrid_job_oar_jobs, wait_oargrid_job_start, \
     get_oargrid_job_nodes, get_oargrid_job_key
 
-from kadeploy import Deployment, Kadeployer, deploy
+from .kadeploy import Deployment, Kadeployer, deploy
 
-from utils import get_kavlan_host_name, G5kAutoPortForwarder
+from .utils import get_kavlan_host_name, G5kAutoPortForwarder
 
-from planning import get_planning, compute_slots, find_first_slot, \
+from .planning import get_planning, compute_slots, find_first_slot, \
     find_max_slot, find_free_slot, get_jobs_specs, distribute_hosts, \
     g5k_charter_time
 
-from api_utils import get_g5k_sites, get_site_clusters, \
+from .api_utils import get_g5k_sites, get_site_clusters, \
     get_cluster_hosts, get_g5k_clusters, get_g5k_hosts, \
-    get_cluster_site, APIConnection, APIGetException, get_host_site, \
+    get_cluster_site, APIConnection, APIException, get_host_site, \
     get_host_cluster, group_hosts, get_resource_attributes, \
-    get_host_attributes, get_cluster_attributes, \
-    get_site_attributes, canonical_host_name
+    get_site_network_equipments, get_host_attributes, get_cluster_attributes, \
+    get_site_attributes, get_network_equipment_attributes, canonical_host_name,\
+    get_network_equipment_site, get_host_shortname, get_host_longname, \
+    get_cluster_network_equipments, get_site_hosts, get_host_network_equipments, \
+    get_hosts_metric
 
-from charter import g5k_charter_time, get_next_charter_period
+from .charter import g5k_charter_time, get_next_charter_period
 
 try:
-    from charter import g5k_charter_remaining
+    from .charter import g5k_charter_remaining
 except:
-    pass # no mysqldb
+    pass # no psycopg2
 
 try:
-    from topology import backbone_graph, site_graph
+    from .topology import g5k_graph, treemap
 except:
     pass # no networkx
