@@ -1,4 +1,4 @@
-# Copyright 2009-2014 INRIA Rhone-Alpes, Service Experimentation et
+# Copyright 2009-2021 INRIA Rhone-Alpes, Service Experimentation et
 # Developpement
 #
 # This file is part of Execo.
@@ -32,12 +32,14 @@ g5k_configuration = {
     'default_frontend' : None,
     'api_uri': "https://api.grid5000.fr/3.0/",
     'api_username': None,
-    'api_additional_args': [],
+    'api_additional_args': {},
+    'api_timeout': 30,
+    'api_verify_ssl_cert': True,
     'oar_job_key_file': None,
-    'oar_mysql_ro_db': 'oar2',
-    'oar_mysql_ro_user': 'oarreader',
-    'oar_mysql_ro_password': 'read',
-    'oar_mysql_ro_port': 3306,
+    'oar_pgsql_ro_db': 'oar2',
+    'oar_pgsql_ro_user': 'oarreader',
+    'oar_pgsql_ro_password': 'read',
+    'oar_pgsql_ro_port': 5432,
     }
 # _ENDOF_ g5k_configuration
 """Global Grid5000 configuration parameters.
@@ -84,10 +86,17 @@ g5k_configuration = {
 
 - ``api_additional_args``: additional arguments to append at the end
   all requests to g5k api. May be used to request the testing branch
-  (use: api_additional_args = ["branch=testing"])
+  (use: api_additional_args = {'branch': 'testing'})
+
+- ``api_timeout``: timeout in seconds of all api requests, before
+  raising an exception.
+
+- ``api_verify_ssl_cert``: If set to false, will disable ssl
+  certificates check for api https requests
 
 - ``oar_job_key_file``: ssh key to use for oar. If defined, takes
   precedence over environment variable OAR_JOB_KEY_FILE.
+
 """
 
 def make_default_oarsh_oarcp_params():
@@ -119,8 +128,10 @@ def make_default_frontend_connection_params():
 default_frontend_connection_params = make_default_frontend_connection_params()
 """Default connection params when connecting to a Grid5000 frontend."""
 
-load_configuration(
-  get_user_config_filename(),
-  ((g5k_configuration, 'g5k_configuration'),
-   (default_frontend_connection_params, 'default_frontend_connection_params'),
-   (default_oarsh_oarcp_params, 'default_oarsh_oarcp_params')))
+__cf = get_user_config_filename()
+if __cf:
+    load_configuration(
+        __cf,
+        ((g5k_configuration, 'g5k_configuration'),
+         (default_frontend_connection_params, 'default_frontend_connection_params'),
+         (default_oarsh_oarcp_params, 'default_oarsh_oarcp_params')))
